/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain.impl;

import de.fhdw.gaming.core.domain.AbstractPlayer;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsFieldState;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsPlayer;

/**
 * Implements {@link VierConnectsPlayer}.
 */
final class VierConnectsPlayerImpl extends AbstractPlayer<VierConnectsPlayer> implements VierConnectsPlayer {

    /**
     * {@code true} if this player uses crosses, or {@code false} if she uses noughts.
     */
    private final boolean usingCrosses;

    /**
     * Creates a Vier Connects player.
     *
     * @param name         The name of the player.
     * @param usingCrosses {@code true} if this player uses crosses, or {@code false} if she uses noughts..
     */
    VierConnectsPlayerImpl(final String name, final boolean usingCrosses) {
        super(name);
        this.usingCrosses = usingCrosses;
    }

    /**
     * Creates a Vier Connects player.
     *
     * @param source The {@link VierConnectsPlayer} to copy.
     */
    VierConnectsPlayerImpl(final VierConnectsPlayer source) {
        super(source);
        this.usingCrosses = source.isUsingCrosses();
    }

    @Override
    public String toString() {
        return String.format(
                "VierConnectsPlayer[name=%s, type=%s, state=%s, outcome=%s]",
                this.getName(),
                this.usingCrosses ? VierConnectsFieldState.CROSS.toString() : VierConnectsFieldState.NOUGHT.toString(),
                this.getState(),
                this.getOutcome());
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof VierConnectsPlayerImpl) {
            final VierConnectsPlayerImpl other = (VierConnectsPlayerImpl) obj;
            return super.equals(obj) && this.usingCrosses == other.usingCrosses;
        }
        return false;
    }

    @Override
    public int hashCode() {
        return super.hashCode() ^ Boolean.hashCode(this.usingCrosses);
    }

    @Override
    public boolean isUsingCrosses() {
        return this.usingCrosses;
    }

    @Override
    public VierConnectsPlayer deepCopy() {
        return new VierConnectsPlayerImpl(this);
    }
}
