/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-VierConnects-core.
 *
 * ipspiel24-VierConnects-core is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-VierConnects-core is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * ipspiel24-VierConnects-core. If not, see <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.ipspiel24.VierConnects.core.domain.impl;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsField;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsFieldState;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsRow;
import de.fhdw.gaming.ipspiel24.VierConnects.core.domain.VierConnectsRowType;

/**
 * Implements {@link VierConnectsRow}.
 */
public final class VierConnectsRowImpl implements VierConnectsRow {

    /**
     * The type of this row.
     */
    private final VierConnectsRowType type;

    /**
     * The zero-based index of this row.
     */
    private final int index;

    /**
     * The fields of this row.
     */
    private final List<? extends VierConnectsField> fields;

    /**
     * Creates a row.
     *
     * @param type   The type of this row.
     * @param index  The zero-based index of this row.
     * @param fields The fields of this row.
     */
    public VierConnectsRowImpl(final VierConnectsRowType type, final int index,
            final List<? extends VierConnectsField> fields) {
        this.type = type;
        this.index = index;
        this.fields = fields;
    }

    @Override
    public boolean equals(final Object obj) {
        if (obj instanceof VierConnectsRowImpl) {
            final VierConnectsRowImpl other = (VierConnectsRowImpl) obj;
            return this.type.equals(other.type) && this.index == other.index && this.fields.equals(other.fields);
        }
        return false;
    }

    @Override
    public int hashCode() {
        return Objects.hash(this.type, this.index, this.fields);
    }

    @Override
    public String toString() {
        return String.format("%s@%d%s", this.type, this.index, this.fields);
    }

    @Override
    public VierConnectsRowType getType() {
        return this.type;
    }

    @Override
    public int getIndex() {
        return this.index;
    }

    @Override
    public List<? extends VierConnectsField> getFields() {
        return this.fields;
    }
//    Old version of TicTacToe
//    @Override
//    public Optional<VierConnectsFieldState> getState() {
//        VierConnectsFieldState lastState = null;
//        for (final VierConnectsField field : this.fields) {
//            if (lastState == null) {
//                lastState = field.getState();
//            } else if (!lastState.equals(field.getState())) {
//                return Optional.empty();
//            }
//        }
//        return Optional.ofNullable(lastState); // may be empty if row is
//    }
    
    @Override
    public Optional<VierConnectsFieldState> getState() {
        VierConnectsFieldState lastState = null;
        int count  = 0;
        for (final VierConnectsField field : this.fields) {
            if (lastState == null) {
                lastState = field.getState();
            } else if (!lastState.equals(field.getState())) {
                lastState = field.getState();
                count = 0;
            }
            count++;
            if (count >= 4 && !lastState.equals(VierConnectsFieldState.EMPTY)) {
                return Optional.ofNullable(lastState);
            }
        }
        return Optional.empty();
    }
}
